use crate::utils::GcliError;

pub fn prompt_password() -> Result<String, GcliError> {
	prompt_password_query("Password")
}

pub fn prompt_password_confirm() -> Result<String, GcliError> {
	prompt_password_query_confirm("Password")
}

pub fn prompt_password_query(query: impl ToString) -> Result<String, GcliError> {
	dialoguer::Password::default()
		.with_prompt(query.to_string())
		.allow_empty_password(true)
		.interact()
		.map_err(|e| GcliError::Input(e.to_string()))
}

pub fn prompt_password_query_confirm(query: impl ToString) -> Result<String, GcliError> {
	dialoguer::Password::new()
		.with_prompt(query.to_string())
		.allow_empty_password(true)
		.with_confirmation(
			format!("Repeat {}", query.to_string()),
			"Error: the values do not match.",
		)
		.interact()
		.map_err(|e| GcliError::Input(e.to_string()))
}

/// Prompt for a (direct) vault name (cannot contain derivation path)
///
/// Also preventing to use '<' and '>' as those are used in the display of
pub fn prompt_vault_name() -> Result<Option<String>, GcliError> {
	let name = dialoguer::Input::new()
		.with_prompt("Name")
		.validate_with({
			move |input: &String| -> Result<(), &str> {
				if input.contains('<') || input.contains('>') || input.contains('/') {
					Err("Name cannot contain characters '<', '>', '/'")
				} else {
					Ok(())
				}
			}
		})
		.allow_empty(true)
		.interact_text()
		.map_err(|e| GcliError::Input(e.to_string()))?;

	let name = if name.trim().is_empty() {
		None
	} else {
		Some(name.trim().to_string())
	};

	Ok(name)
}

/// Prompt for a derivation path
pub fn prompt_vault_derivation_path() -> Result<String, GcliError> {
	let path = dialoguer::Input::new()
		.with_prompt("Derivation path")
		.validate_with({
			move |input: &String| -> Result<(), &str> {
				if !input.starts_with("/") {
					Err("derivation path needs to start with one or more '/'")
				} else {
					Ok(())
				}
			}
		})
		.allow_empty(false)
		.interact_text()
		.map_err(|e| GcliError::Input(e.to_string()))?;

	Ok(path)
}

pub fn confirm_action(query: impl ToString) -> Result<bool, GcliError> {
	dialoguer::Confirm::new()
		.with_prompt(query.to_string())
		//.default(false)
		.interact()
		.map_err(|e| GcliError::Input(e.to_string()))
}
